@@ -57,7 +57,7 @@ class MatchType(Enum):
     MAPPING = {
         NONE: [],
         BASENAME: ['--match-basename'],
-        EXTENSION: ['--match-with-extension'],
+        EXTENSION: ['--match-extension'],
         WITHOUT_EXTENSION: ['--match-without-extension']
     }
 
@@ -167,7 +167,7 @@ def _create_rmlint_process(
             map_cfg(CrossMountType,
                     cfg.get_boolean('traverse-cross-mounts')),
             map_cfg(AlgorithmType,
-                    cfg.get_boolean('computation-algorithm'))
+                    cfg.get_enum('computation-algorithm'))
         ]
 
         # Flatten list:
@@ -253,7 +253,7 @@ class Runner(GObject.Object):
 
     def on_process_termination(self, process, result):
         """Called once GSuprocess sees its child die."""
-        # We dont emit process-finished yet here.
+        # We don't emit process-finished yet here.
         # We still might get some items from the stream.
         # Call process-finished once we hit EOF.
 
