@@ -7,7 +7,7 @@ This code here is supposed to be drop-in able
 into other projects if necessary.
 
 Also included: Various specialised GtkCellRenderer implementation.
-Reason for special derivates are given in the respective class.
+Reason for special derivatives are given in the respective class.
 """
 
 # Stdlib:
@@ -499,7 +499,7 @@ class CellRendererCount(Gtk.CellRendererText):
         positive numbers as objects in a directory.
         """
         count = self.get_property('count')
-        is_plural = abs(count) is not 1
+        is_plural = abs(count) != 1
 
         if count > 0:
             name = 'Objects' if is_plural else 'Object'
@@ -761,7 +761,7 @@ EXPONENTS = {
     'Kilobyte': 1,
     'Megabyte': 2,
     'Gigabyte': 3,
-    'Terrabye': 4,
+    'Terabyte': 4,
     'Petabyte': 5
 }
 
@@ -803,12 +803,12 @@ class FileSizeSpinButton(Gtk.Box):
 
     def get_bytes(self):
         """Get the current number of displayed bytes"""
-        return self._last_val * (1024 ** self._curr_exp)
+        return int(self._last_val * (1024 ** self._curr_exp))
 
     def set_bytes(self, size):
         """Set the current number of displayed bytes"""
         # Find out what unit to use:
-        if size is 0:
+        if size == 0:
             exponent = 1
         else:
             exponent = math.floor(math.log(size, 1024))
@@ -852,7 +852,7 @@ class FileSizeSpinButton(Gtk.Box):
         label = self._units.get_selected_choice()
         exponent = EXPONENTS.get(label, MAX_EXPONENT)
         self._set_exponent(exponent)
-        self.emit('value-changed', self.get_bytes())
+        self.on_value_changed(self._entry)
 
 
 class FileSizeRange(Gtk.Grid):
@@ -888,7 +888,7 @@ class FileSizeRange(Gtk.Grid):
 
     @min_value.setter
     def min_value(self, val):
-        """Set the minum value."""
+        """Set the minimum value."""
         self._min_wdgt.set_bytes(val)
 
     @property
