@@ -32,8 +32,8 @@ You can contact the author at :
 */
 
 /**************************************
-*  Tuning parameters
-**************************************/
+ *  Tuning parameters
+ **************************************/
 /* Unaligned memory access is automatically enabled for "common" CPU, such as x86.
  * For others CPU, the compiler will be more cautious, and insert extra code to ensure
  * aligned access is respected.
@@ -70,36 +70,27 @@ You can contact the author at :
 #define XXH_FORCE_NATIVE_FORMAT 0
 
 /**************************************
-*  Compiler Specific Options
-***************************************/
+ *  Compiler Specific Options
+ ***************************************/
 #ifdef _MSC_VER                 /* Visual Studio */
 #pragma warning(disable : 4127) /* disable: C4127: conditional expression is constant */
-#define FORCE_INLINE static __forceinline
-#else
-#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L /* C99 */
-#ifdef __GNUC__
-#define FORCE_INLINE static inline __attribute__((always_inline))
-#else
-#define FORCE_INLINE static inline
-#endif
-#else
-#define FORCE_INLINE static
-#endif /* __STDC_VERSION__ */
 #endif
 
+#include "../../config.h" // INLINE
+
 /**************************************
-*  Includes & Memory related functions
-***************************************/
+ *  Includes & Memory related functions
+ ***************************************/
 #include "xxhash.h"
+
 /* Modify the local functions below should you wish to use some other memory routines */
 /* for malloc(), free() */
-#include <stdlib.h>
-static void* XXH_malloc(size_t s) {
-    return malloc(s);
-}
-static void XXH_free(void* p) {
-    free(p);
-}
+
+#include <glib.h>   // g_slice_alloc, g_slice_free1
+#define XXH_new(type) g_slice_alloc(sizeof(type));
+/* Note that "*pointer" must provide the type of the full object */
+#define XXH_delete(pointer) g_slice_free1(sizeof(*pointer), pointer)
+
 /* for memcpy() */
 #include <string.h>
 static void* XXH_memcpy(void* dest, const void* src, size_t size) {
@@ -107,8 +98,8 @@ static void* XXH_memcpy(void* dest, const void* src, size_t size) {
 }
 
 /**************************************
-*  Basic Types
-***************************************/
+ *  Basic Types
+ ***************************************/
 #if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901L /* C99 */
 #include <stdint.h>
 typedef uint8_t BYTE;
@@ -137,8 +128,8 @@ static U64 XXH_read64(const void* memPtr) {
 }
 
 /******************************************
-*  Compiler-specific Functions and Macros
-******************************************/
+ *  Compiler-specific Functions and Macros
+ ******************************************/
 #define GCC_VERSION (__GNUC__ * 100 + __GNUC_MINOR__)
 
 /* Note : although _rotl exists for minGW (GCC under windows), performance seems poor */
@@ -170,8 +161,8 @@ static U64 XXH_swap64(U64 x) {
 #endif
 
 /***************************************
-*  Architecture Macros
-***************************************/
+ *  Architecture Macros
+ ***************************************/
 typedef enum { XXH_bigEndian = 0, XXH_littleEndian = 1 } XXH_endianess;
 #ifndef XXH_CPU_LITTLE_ENDIAN /* XXH_CPU_LITTLE_ENDIAN can be defined externally, for \
                                  example using a compiler switch */
@@ -180,11 +171,11 @@ static const int one = 1;
 #endif
 
 /*****************************
-*  Memory reads
-*****************************/
+ *  Memory reads
+ *****************************/
 typedef enum { XXH_aligned, XXH_unaligned } XXH_alignment;
 
-FORCE_INLINE U32 XXH_readLE32_align(const void* ptr, XXH_endianess endian,
+static INLINE U32 XXH_readLE32_align(const void* ptr, XXH_endianess endian,
                                     XXH_alignment align) {
     if(align == XXH_unaligned)
         return endian == XXH_littleEndian ? XXH_read32(ptr) : XXH_swap32(XXH_read32(ptr));
@@ -193,11 +184,11 @@ FORCE_INLINE U32 XXH_readLE32_align(const void* ptr, XXH_endianess endian,
                                           : XXH_swap32(*(const U32*)ptr);
 }
 
-FORCE_INLINE U32 XXH_readLE32(const void* ptr, XXH_endianess endian) {
+static INLINE U32 XXH_readLE32(const void* ptr, XXH_endianess endian) {
     return XXH_readLE32_align(ptr, endian, XXH_unaligned);
 }
 
-FORCE_INLINE U64 XXH_readLE64_align(const void* ptr, XXH_endianess endian,
+static INLINE U64 XXH_readLE64_align(const void* ptr, XXH_endianess endian,
                                     XXH_alignment align) {
     if(align == XXH_unaligned)
         return endian == XXH_littleEndian ? XXH_read64(ptr) : XXH_swap64(XXH_read64(ptr));
@@ -206,21 +197,21 @@ FORCE_INLINE U64 XXH_readLE64_align(const void* ptr, XXH_endianess endian,
                                           : XXH_swap64(*(const U64*)ptr);
 }
 
-FORCE_INLINE U64 XXH_readLE64(const void* ptr, XXH_endianess endian) {
+static INLINE U64 XXH_readLE64(const void* ptr, XXH_endianess endian) {
     return XXH_readLE64_align(ptr, endian, XXH_unaligned);
 }
 
 /***************************************
-*  Macros
-***************************************/
+ *  Macros
+ ***************************************/
 #define XXH_STATIC_ASSERT(c)                      \
     {                                             \
         enum { XXH_static_assert = 1 / (!!(c)) }; \
     } /* use only *after* variable declarations */
 
 /***************************************
-*  Constants
-***************************************/
+ *  Constants
+ ***************************************/
 #define PRIME32_1 2654435761U
 #define PRIME32_2 2246822519U
 #define PRIME32_3 3266489917U
@@ -234,9 +225,9 @@ FORCE_INLINE U64 XXH_readLE64(const void* ptr, XXH_endianess endian) {
 #define PRIME64_5 2870177450012600261ULL
 
 /*****************************
-*  Simple Hash Functions
-*****************************/
-FORCE_INLINE U32 XXH32_endian_align(const void* input, size_t len, U32 seed,
+ *  Simple Hash Functions
+ *****************************/
+static INLINE U32 XXH32_endian_align(const void* input, size_t len, U32 seed,
                                     XXH_endianess endian, XXH_alignment align) {
     const BYTE* p = (const BYTE*)input;
     const BYTE* bEnd = p + len;
@@ -333,7 +324,7 @@ unsigned XXH32(const void* input, size_t len, unsigned seed) {
 #endif
 }
 
-FORCE_INLINE U64 XXH64_endian_align(const void* input, size_t len, U64 seed,
+static INLINE U64 XXH64_endian_align(const void* input, size_t len, U64 seed,
                                     XXH_endianess endian, XXH_alignment align) {
     const BYTE* p = (const BYTE*)input;
     const BYTE* bEnd = p + len;
@@ -465,8 +456,8 @@ unsigned long long XXH64(const void* input, size_t len, unsigned long long seed)
 }
 
 /****************************************************
-*  Advanced Hash Functions
-****************************************************/
+ *  Advanced Hash Functions
+ ****************************************************/
 
 /*** Allocation ***/
 typedef struct {
@@ -495,10 +486,10 @@ XXH32_state_t* XXH32_createState(void) {
     XXH_STATIC_ASSERT(sizeof(XXH32_state_t) >=
                       sizeof(XXH_istate32_t)); /* A compilation error here means
                                                   XXH32_state_t is not large enough */
-    return (XXH32_state_t*)XXH_malloc(sizeof(XXH32_state_t));
+    return (XXH32_state_t*)XXH_new(XXH32_state_t);
 }
 XXH_errorcode XXH32_freeState(XXH32_state_t* statePtr) {
-    XXH_free(statePtr);
+    XXH_delete(statePtr);
     return XXH_OK;
 }
 
@@ -506,10 +497,10 @@ XXH64_state_t* XXH64_createState(void) {
     XXH_STATIC_ASSERT(sizeof(XXH64_state_t) >=
                       sizeof(XXH_istate64_t)); /* A compilation error here means
                                                   XXH64_state_t is not large enough */
-    return (XXH64_state_t*)XXH_malloc(sizeof(XXH64_state_t));
+    return (XXH64_state_t*)XXH_new(XXH64_state_t);
 }
 XXH_errorcode XXH64_freeState(XXH64_state_t* statePtr) {
-    XXH_free(statePtr);
+    XXH_delete(statePtr);
     return XXH_OK;
 }
 
@@ -539,7 +530,7 @@ XXH_errorcode XXH64_reset(XXH64_state_t* state_in, unsigned long long seed) {
     return XXH_OK;
 }
 
-FORCE_INLINE XXH_errorcode XXH32_update_endian(XXH32_state_t* state_in, const void* input,
+static INLINE XXH_errorcode XXH32_update_endian(XXH32_state_t* state_in, const void* input,
                                                size_t len, XXH_endianess endian) {
     XXH_istate32_t* state = (XXH_istate32_t*)state_in;
     const BYTE* p = (const BYTE*)input;
@@ -634,7 +625,7 @@ XXH_errorcode XXH32_update(XXH32_state_t* state_in, const void* input, size_t le
         return XXH32_update_endian(state_in, input, len, XXH_bigEndian);
 }
 
-FORCE_INLINE U32 XXH32_digest_endian(const XXH32_state_t* state_in,
+static INLINE U32 XXH32_digest_endian(const XXH32_state_t* state_in,
                                      XXH_endianess endian) {
     const XXH_istate32_t* state = (const XXH_istate32_t*)state_in;
     const BYTE* p = (const BYTE*)state->mem32;
@@ -680,7 +671,7 @@ U32 XXH32_digest(const XXH32_state_t* state_in) {
         return XXH32_digest_endian(state_in, XXH_bigEndian);
 }
 
-FORCE_INLINE XXH_errorcode XXH64_update_endian(XXH64_state_t* state_in, const void* input,
+static INLINE XXH_errorcode XXH64_update_endian(XXH64_state_t* state_in, const void* input,
                                                size_t len, XXH_endianess endian) {
     XXH_istate64_t* state = (XXH_istate64_t*)state_in;
     const BYTE* p = (const BYTE*)input;
@@ -775,7 +766,7 @@ XXH_errorcode XXH64_update(XXH64_state_t* state_in, const void* input, size_t le
         return XXH64_update_endian(state_in, input, len, XXH_bigEndian);
 }
 
-FORCE_INLINE U64 XXH64_digest_endian(const XXH64_state_t* state_in,
+static INLINE U64 XXH64_digest_endian(const XXH64_state_t* state_in,
                                      XXH_endianess endian) {
     const XXH_istate64_t* state = (const XXH_istate64_t*)state_in;
     const BYTE* p = (const BYTE*)state->mem64;
