@@ -176,7 +176,7 @@ static void rm_cmd_start_gui(int argc, const char **argv) {
         return;
     }
 
-    if(write(bootstrap_fd, RM_PY_BOOTSTRAP, sizeof(RM_PY_BOOTSTRAP)) < 0) {
+    if(write(bootstrap_fd, RM_PY_BOOTSTRAP, strlen(RM_PY_BOOTSTRAP)) < 0) {
         rm_log_warning_line("Could not bootstrap gui: Unable to write to tempfile: %s",
                             g_strerror(errno));
         return;
@@ -204,7 +204,7 @@ static void rm_cmd_start_gui(int argc, const char **argv) {
             rm_log_warning("\n");
             rm_log_error_line("%s %d", g_strerror(errno), errno == ENOENT);
         } else {
-            /* This is not reached anymore when execve suceeded */
+            /* This is not reached anymore when execve succeeded */
             break;
         }
 
@@ -351,7 +351,7 @@ static RmOff rm_cmd_size_string_to_bytes(const char *size_spec, GError **error)
             return 0;
         }
 
-        if(fraction_num == ULONG_MAX && errno == ERANGE) {
+        if(fraction_num == ULLONG_MAX && errno == ERANGE) {
             g_set_error(error, RM_ERROR_QUARK, 0, _("Fraction is too big for uint64"));
             return 0;
         }
@@ -371,7 +371,7 @@ static RmOff rm_cmd_size_string_to_bytes(const char *size_spec, GError **error)
             return 0;
         }
 
-        if(base_size == ULONG_MAX && errno == ERANGE) {
+        if(base_size == ULLONG_MAX && errno == ERANGE) {
             g_set_error(error, RM_ERROR_QUARK, 0, _("Size is too big for uint64"));
             return 0;
         }
@@ -380,13 +380,15 @@ static RmOff rm_cmd_size_string_to_bytes(const char *size_spec, GError **error)
     RmOff result = base_size;
     if(need_multiply) {
         // Only multiply if we really have to.
-        result = (result + fraction) * size_factor;
+        double fres = (result + fraction) * size_factor;
 
         // Check if an overflow happened during multiplication.
-        if(result < base_size) {
-            g_set_error(error, RM_ERROR_QUARK, 0, _("Size factor would overflow size (max. 2**64 allowed)"));
+        if(fres > nextafter((double)ULLONG_MAX, 0)) {
+            g_set_error(error, RM_ERROR_QUARK, 0,
+                        _("Size factor would overflow size (max. 2**64 allowed)"));
             return 0;
         }
+        result = fres;
     }
 
     return result;
@@ -625,6 +627,8 @@ static void rm_cmd_parse_clamp_option(RmSession *session, const char *string,
             session->cfg->skip_end_offset = offset;
         }
     }
+
+    session->cfg->clamp_is_used = true;
 }
 
 static gboolean rm_cmd_parse_partial_hidden(_UNUSED const char *option_name,
@@ -886,7 +890,7 @@ static gboolean rm_cmd_parse_timestamp_file(const char *option_name,
 
         fclose(stamp_file);
     } else {
-        /* Cannot read a stamp file, assume we gonna creae it. */
+        /* Cannot read a stamp file, assume we have to create it. */
         plain = false;
         success = true;
         rm_log_info_line(_("No stamp file at `%s`, will create one after this run."), timestamp_path);
@@ -1331,9 +1335,13 @@ static bool rm_cmd_set_paths(RmSession *session, char **paths) {
 }
 
 static bool rm_cmd_set_outputs(RmSession *session, GError **error) {
+    if(session->cfg->run_equal_mode) {
+        return true;
+    }
+
     if(session->output_cnt[0] >= 0 && session->output_cnt[1] >= 0) {
         g_set_error(error, RM_ERROR_QUARK, 0,
-                    _("Specifiyng both -o and -O is not allowed"));
+                    _("Specifying both -o and -O is not allowed"));
         return false;
     } else if(session->output_cnt[0] < 0 && session->cfg->progress_enabled == false) {
         rm_cmd_set_default_outputs(session);
@@ -1601,8 +1609,8 @@ bool rm_cmd_parse_args(int argc, char **argv, RmSession *session) {
         ); goto cleanup;
     }
 
-    if(cfg->dedupe) {
-        /* dedupe session; regular rmlint configs are ignored */
+    if(cfg->dedupe || cfg->is_reflink) {
+        /* dedupe or is-reflink session; regular rmlint configs are ignored */
         goto cleanup;
     }
 
@@ -1755,7 +1763,7 @@ int rm_cmd_main(RmSession *session) {
         g_assert(cfg->cache_file_structs);
 
         /* Currently we cannot use -D and the cloning on btrfs, since this assumes the same layout
-         * on two dupicate directories which is likely not a valid assumption.
+         * on two duplicate directories which is likely not a valid assumption.
          * Emit a warning if the raw -D is used in conjunction with that.
          * */
         const char *handler_key = rm_fmt_get_config_value(session->formats, "sh", "handler");
