@@ -16,8 +16,8 @@
 *
 * Authors:
 *
-*  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
-*  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+*  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+*  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
 *
 * Hosted on http://github.com/sahib/rmlint
 */
@@ -65,13 +65,14 @@ RmFile *rm_file_new(struct RmSession *session, const char *path, RmStat *statp,
     self->path_depth = rm_util_path_depth(path);
     self->file_size = 0;
     self->actual_file_size = 0;
+    self->n_children = 0;
 
     self->inode = statp->st_ino;
     self->dev = statp->st_dev;
     self->mtime = rm_sys_stat_mtime_float(statp);
     self->is_new = (self->mtime >= cfg->min_mtime);
 
-    if(type == RM_LINT_TYPE_DUPE_CANDIDATE) {
+    if(type == RM_LINT_TYPE_DUPE_CANDIDATE || type == RM_LINT_TYPE_PART_OF_DIRECTORY) {
         if(cfg->use_absolute_end_offset) {
             self->file_size = CLAMP(actual_file_size, 1, cfg->skip_end_offset);
         } else {
@@ -99,7 +100,7 @@ RmFile *rm_file_new(struct RmSession *session, const char *path, RmStat *statp,
 }
 
 void rm_file_set_path(RmFile *file, char *path) {
-    file->folder = rm_trie_insert(&file->session->cfg->file_trie, path, NULL);
+    file->folder = rm_trie_insert(&file->session->cfg->file_trie, path, file);
 }
 
 void rm_file_build_path(RmFile *file, char *buf) {
@@ -108,6 +109,26 @@ void rm_file_build_path(RmFile *file, char *buf) {
     rm_trie_build_path(&file->session->cfg->file_trie, file->folder, buf, PATH_MAX);
 }
 
+RmFile *rm_file_copy(RmFile *file) {
+    g_assert(file);
+
+    RmFile *copy  = g_slice_new0(RmFile);
+    memcpy(copy, file, sizeof(RmFile));
+
+    /* Only reset/copy the complex fields */
+    copy->digest = rm_digest_copy(file->digest);
+    copy->ext_cksum = g_strdup(file->ext_cksum);
+
+    copy->cluster = NULL;
+    copy->hardlinks = NULL;
+    copy->shred_group = NULL;
+    copy->signal = NULL;
+    copy->parent_dir = NULL;
+    copy->n_children = 0;
+
+	return copy;
+}
+
 void rm_file_destroy(RmFile *file) {
     if(file->hardlinks) {
         g_queue_remove(file->hardlinks, file);
@@ -137,6 +158,7 @@ static const char *LINT_TYPES[] = {[RM_LINT_TYPE_UNKNOWN] = "",
                                    [RM_LINT_TYPE_EMPTY_FILE] = "emptyfile",
                                    [RM_LINT_TYPE_DUPE_CANDIDATE] = "duplicate_file",
                                    [RM_LINT_TYPE_DUPE_DIR_CANDIDATE] = "duplicate_dir",
+                                   [RM_LINT_TYPE_PART_OF_DIRECTORY] = "part_of_directory",
                                    [RM_LINT_TYPE_UNIQUE_FILE] = "unique_file"};
 
 const char *rm_file_lint_type_to_string(RmLintType type) {
