@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  *
@@ -33,7 +33,7 @@
 
 /* A group of output files.
  * These are only created when caching to the end of the run is requested.
- * Otherwise, files are directly outputed and not stored in groups.
+ * Otherwise, files are directly outputted and not stored in groups.
  */
 typedef struct RmFmtGroup {
     GQueue files;
@@ -46,34 +46,10 @@ static RmFmtGroup *rm_fmt_group_new(void) {
     return self;
 }
 
-static void rm_fmt_group_destroy(RmFmtTable *self, RmFmtGroup *group) {
-    RmCfg *cfg = self->session->cfg;
-
-    /* Special case: treemerge.c has to manage memory itself,
-     *               since it omits some files or may even print them twice.
-     */
-    bool needs_free = true;
-    if(cfg->merge_directories) {
-        needs_free = false;
-    }
-
-    /* Unique files are not fed to treemerge.c,
-     * therefore we need to free them here.
-     * Can't free them earlier, since '-o uniques' still need
-     * to output them if requested.
-     * */
-    if(needs_free == false && group->files.length == 1) {
-        RmFile *file = (RmFile *)group->files.head->data;
-        if(file && file->lint_type == RM_LINT_TYPE_UNIQUE_FILE) {
-            // needs_free = true; /* TODO: this is a temporary workaround for double-free of digest with treemerge */
-        }
-    }
-
-    if(needs_free) {
-        for(GList *iter = group->files.head; iter; iter = iter->next) {
-            RmFile *file = iter->data;
-            rm_file_destroy(file);
-        }
+static void rm_fmt_group_destroy(_UNUSED RmFmtTable *self, RmFmtGroup *group) {
+    for(GList *iter = group->files.head; iter; iter = iter->next) {
+        RmFile *file = iter->data;
+        rm_file_destroy(file);
     }
 
     g_queue_clear(&group->files);
@@ -82,7 +58,6 @@ static void rm_fmt_group_destroy(RmFmtTable *self, RmFmtGroup *group) {
 
 static void rm_fmt_handler_free(RmFmtHandler *handler) {
     g_assert(handler);
-
     g_free(handler->path);
     g_free(handler);
 }
@@ -195,6 +170,35 @@ void rm_fmt_remove_by_name(RmFmtTable *self, char *name) {
     g_hash_table_remove(self->handler_set, name);
 }
 
+// NOTE: This is the same as g_date_time_format_iso8601.
+// It is only copied here because it is only available since
+// GLib 2.62. Remove this in a few years (written as of end 2019)
+gchar *rm_date_time_format_iso8601(GDateTime *datetime) {
+    GString *outstr = NULL;
+    gchar *main_date = NULL;
+    gint64 offset;
+
+    /* Main date and time. */
+    main_date = g_date_time_format(datetime, "%Y-%m-%dT%H:%M:%S");
+    outstr = g_string_new(main_date);
+    g_free(main_date);
+
+    /* Timezone. Format it as `%:::z` unless the offset is zero, in which case
+     * we can simply use `Z`. */
+    offset = g_date_time_get_utc_offset(datetime);
+
+    if(offset == 0) {
+        g_string_append_c(outstr, 'Z');
+    } else {
+        gchar *time_zone = g_date_time_format(datetime, "%:::z");
+        g_string_append (outstr, time_zone);
+        g_free (time_zone);
+    }
+
+    return g_string_free(outstr, FALSE);
+}
+
+
 void rm_fmt_clear(RmFmtTable *self) {
     if(rm_fmt_len(self) <= 0) {
         return;
@@ -208,12 +212,12 @@ void rm_fmt_clear(RmFmtTable *self) {
 }
 
 void rm_fmt_backup_old_result_file(RmFmtTable *self, const char *old_path) {
-    if(self->first_backup_timestamp.tv_sec == 0L) {
-        g_get_current_time(&self->first_backup_timestamp);
+    if(self->first_backup_timestamp == NULL) {
+        self->first_backup_timestamp = g_date_time_new_now_utc();
     }
 
     char *new_path = NULL;
-    char *timestamp = g_time_val_to_iso8601(&self->first_backup_timestamp);
+    char *timestamp = rm_date_time_format_iso8601(self->first_backup_timestamp);
 
     // Split the extension, if possible and place it before the timestamp suffix.
     char *extension = g_utf8_strrchr(old_path, -1, '.');
@@ -227,7 +231,7 @@ void rm_fmt_backup_old_result_file(RmFmtTable *self, const char *old_path) {
 
 
     rm_log_debug_line(_("Old result `%s` already exists."), old_path);
-    rm_log_debug_line(_("Moving old file to `%s`. Use --no-backup to disable this."), new_path);
+    rm_log_debug_line(_("Moving old file to `%s`. Leave out --backup to disable this."), new_path);
 
     if(rename(old_path, new_path) < 0) {
         rm_log_perror(_("failed to rename old result file"));
@@ -291,7 +295,7 @@ bool rm_fmt_add(RmFmtTable *self, const char *handler_name, const char *path) {
         needs_full_path = true;
         if(access(path, F_OK) == 0) {
             file_existed_already = true;
-            if(!self->session->cfg->no_backup) {
+            if(self->session->cfg->backup) {
                 rm_fmt_backup_old_result_file(self, path);
             }
         }
@@ -357,21 +361,29 @@ static gint rm_fmt_rank_size(const RmFmtGroup *ga, const RmFmtGroup *gb) {
     return SIGN_DIFF(sa, sb);
 }
 
+static int rm_lint_type_order[] = {
+    /* Other types have a prio of 0 by default */
+    [RM_LINT_TYPE_PART_OF_DIRECTORY]  = 1,
+    [RM_LINT_TYPE_DUPE_DIR_CANDIDATE] = 2,
+    [RM_LINT_TYPE_DUPE_CANDIDATE]     = 3,
+};
+
 static gint rm_fmt_rank(const RmFmtGroup *ga, const RmFmtGroup *gb, RmFmtTable *self) {
     const char *rank_order = self->session->cfg->rank_criteria;
 
     RmFile *fa = ga->files.head->data;
     RmFile *fb = gb->files.head->data;
 
-    if(fa->lint_type != RM_LINT_TYPE_DUPE_CANDIDATE &&
-       fa->lint_type != RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
-        return -1;
+    RM_DEFINE_PATH(fa);
+    RM_DEFINE_PATH(fb);
+
+    int fa_order = rm_lint_type_order[fa->lint_type];
+    int fb_order = rm_lint_type_order[fb->lint_type];
+    if(fa_order != fb_order) {
+        return fa_order - fb_order;
     }
 
-    if(fb->lint_type != RM_LINT_TYPE_DUPE_CANDIDATE &&
-       fb->lint_type != RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
-        return +1;
-    }
+    /*  Sort by ranking */
 
     for(int i = 0; rank_order[i]; ++i) {
         gint r = 0;
@@ -410,7 +422,7 @@ void rm_fmt_flush(RmFmtTable *self) {
         return;
     }
 
-    if(*(cfg->rank_criteria)) {
+    if(*(cfg->rank_criteria) || cfg->replay) {
         g_queue_sort(&self->groups, (GCompareDataFunc)rm_fmt_rank, self);
     }
 
@@ -442,14 +454,17 @@ void rm_fmt_close(RmFmtTable *self) {
     g_hash_table_unref(self->handler_set);
     g_queue_free(self->handler_order);
     g_rec_mutex_clear(&self->state_mtx);
+
+    if(self->first_backup_timestamp) {
+        g_date_time_unref(self->first_backup_timestamp);
+    }
+
     g_slice_free(RmFmtTable, self);
 }
 
-void rm_fmt_write(RmFile *result, RmFmtTable *self, gint64 twin_count) {
+void rm_fmt_write(RmFile *result, RmFmtTable *self) {
     bool direct = !(self->session->cfg->cache_file_structs);
 
-    result->twin_count = twin_count;
-
     if(direct) {
         rm_fmt_write_impl(result, self);
     } else {
