@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  *
@@ -27,6 +27,7 @@
 #include "../formats.h"
 #include "../preprocess.h"
 #include "../utilities.h"
+#include "../treemerge.h"
 
 #include <glib.h>
 #include <stdio.h>
@@ -206,6 +207,9 @@ static void rm_fmt_head(RmSession *session, _UNUSED RmFmtHandler *parent, FILE *
                 rm_fmt_json_sep(self, out);
                 rm_fmt_json_key_int(out, "hash_seed", session->hash_seed);
             }
+
+            rm_fmt_json_sep(self, out);
+            rm_fmt_json_key_bool(out, "merge_directories", session->cfg->merge_directories);
         }
         rm_fmt_json_close(self, out);
     }
@@ -263,6 +267,16 @@ static void rm_fmt_elem(RmSession *session, _UNUSED RmFmtHandler *parent, FILE *
                 return;
             }
         }
+
+        if(session->cfg->keep_all_tagged && !file->is_prefd) {
+            /* don't list 'untagged' files as unique */
+            file->is_original = false;
+        } else if(session->cfg->keep_all_untagged && file->is_prefd) {
+            /* don't list 'tagged' files as unique */
+            file->is_original = false;
+        } else {
+            file->is_original = true;
+        }
     }
 
     char *checksum_str = NULL;
@@ -309,21 +323,34 @@ static void rm_fmt_elem(RmSession *session, _UNUSED RmFmtHandler *parent, FILE *
 
         rm_fmt_json_key_unsafe(out, "path", file_path);
         rm_fmt_json_sep(self, out);
-        if(file->lint_type != RM_LINT_TYPE_UNIQUE_FILE) {
-            rm_fmt_json_key_int(out, "size", file->actual_file_size);
+        rm_fmt_json_key_int(out, "size", file->actual_file_size);
+        rm_fmt_json_sep(self, out);
+        rm_fmt_json_key_int(out, "depth", file->depth);
+        rm_fmt_json_sep(self, out);
+        rm_fmt_json_key_int(out, "inode", file->inode);
+        rm_fmt_json_sep(self, out);
+        rm_fmt_json_key_int(out, "disk_id", file->dev);
+        rm_fmt_json_sep(self, out);
+        rm_fmt_json_key_bool(out, "is_original", file->is_original);
+        rm_fmt_json_sep(self, out);
+
+        if(file->lint_type == RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
+            rm_fmt_json_key_int(out, "n_children", file->n_children);
             rm_fmt_json_sep(self, out);
+        }
+
+        if(file->lint_type != RM_LINT_TYPE_UNIQUE_FILE) {
             if(file->twin_count >= 0) {
                 rm_fmt_json_key_int(out, "twins", file->twin_count);
                 rm_fmt_json_sep(self, out);
             }
-            rm_fmt_json_key_int(out, "depth", file->depth);
-            rm_fmt_json_sep(self, out);
-            rm_fmt_json_key_int(out, "inode", file->inode);
-            rm_fmt_json_sep(self, out);
-            rm_fmt_json_key_int(out, "disk_id", file->dev);
-            rm_fmt_json_sep(self, out);
-            rm_fmt_json_key_bool(out, "is_original", file->is_original);
-            rm_fmt_json_sep(self, out);
+
+
+			if(file->lint_type == RM_LINT_TYPE_PART_OF_DIRECTORY && file->parent_dir) {
+				rm_fmt_json_key_unsafe(out, "parent_path", rm_directory_get_dirname(file->parent_dir));
+				rm_fmt_json_sep(self, out);
+
+			}
 
             if(session->cfg->find_hardlinked_dupes) {
                 RmFile *hardlink_head = RM_FILE_HARDLINK_HEAD(file);
@@ -343,6 +370,7 @@ static void rm_fmt_elem(RmSession *session, _UNUSED RmFmtHandler *parent, FILE *
                 }
             }
         }
+
         rm_fmt_json_key_float(out, "mtime", file->mtime);
     }
     rm_fmt_json_close(self, out);
