@@ -20,8 +20,6 @@ fi
 USER='%s'
 GROUP='%s'
 
-STAMPFILE=$(mktemp 'rmlint.XXXXXXXX.stamp')
-
 # Set to true on -n
 DO_DRY_RUN=
 
@@ -40,6 +38,9 @@ DO_DELETE_EMPTY_DIRS=
 # Set to true on -k
 DO_KEEP_DIR_TIMESTAMPS=
 
+# Set to true on -i
+DO_ASK_BEFORE_DELETE=
+
 ##################################
 # GENERAL LINT HANDLER FUNCTIONS #
 ##################################
@@ -150,7 +151,7 @@ original_check() {
 
     # Check they are not the exact same file (hardlinks allowed):
     if [ "$1" = "$2" ]; then
-        echo "${COL_RED}^^^^^^ Error: original and duplicate point to the *same* path - cancelling.....{COL_RESET}"
+        echo "${COL_RED}^^^^^^ Error: original and duplicate point to the *same* path - cancelling.....${COL_RESET}"
         return 1
     fi
 
@@ -160,6 +161,7 @@ original_check() {
     else
         if [ "$(check_for_equality "$1" "$2")" -ne "0" ]; then
             echo "${COL_RED}^^^^^^ Error: files no longer identical - cancelling.....${COL_RESET}"
+            return 1
         fi
     fi
 }
@@ -255,19 +257,21 @@ remove_cmd() {
     echo "${COL_YELLOW}Deleting: ${COL_RESET}$1"
     if original_check "$1" "$2"; then
         if [ -z "$DO_DRY_RUN" ]; then
-            if [ ! -z "$DO_KEEP_DIR_TIMESTAMPS" ]; then
-                touch -r "$(dirname $1)" "$STAMPFILE"
+            if [ -n "$DO_KEEP_DIR_TIMESTAMPS" ]; then
+                touch -r "$(dirname "$1")" "$STAMPFILE"
             fi
-
-            rm -rf "$1"
-
-            if [ ! -z "$DO_KEEP_DIR_TIMESTAMPS" ]; then
+            if [ -n "$DO_ASK_BEFORE_DELETE" ]; then
+              rm -ri "$1"
+            else
+              rm -rf "$1"
+            fi
+            if [ -n "$DO_KEEP_DIR_TIMESTAMPS" ]; then
                 # Swap back old directory timestamp:
-                touch -r "$STAMPFILE" "$(dirname $1)"
+                touch -r "$STAMPFILE" "$(dirname "$1")"
                 rm "$STAMPFILE"
             fi
 
-            if [ ! -z "$DO_DELETE_EMPTY_DIRS" ]; then
+            if [ -n "$DO_DELETE_EMPTY_DIRS" ]; then
                 DIR=$(dirname "$1")
                 while [ ! "$(ls -A "$DIR")" ]; do
                     print_progress_prefix 0
@@ -326,13 +330,14 @@ OPTIONS:
   -c   Clean up empty directories while deleting duplicates.
   -q   Do not show progress.
   -k   Keep the timestamp of directories when removing duplicates.
+  -i   Ask before deleting each file
 EOF
 }
 
 DO_REMOVE=
 DO_ASK=
 
-while getopts "dhxnrpqck" OPTION
+while getopts "dhxnrpqcki" OPTION
 do
   case $OPTION in
      h)
@@ -349,6 +354,7 @@ do
        DO_DRY_RUN=true
        DO_REMOVE=false
        DO_ASK=false
+       DO_ASK_BEFORE_DELETE=false
        ;;
      r)
        DO_CLONE_READONLY=true
@@ -364,6 +370,10 @@ do
        ;;
      k)
        DO_KEEP_DIR_TIMESTAMPS=true
+       STAMPFILE=$(mktemp 'rmlint.XXXXXXXX.stamp')
+       ;;
+     i)
+       DO_ASK_BEFORE_DELETE=true
        ;;
      *)
        usage
@@ -382,7 +392,7 @@ then
   ask
 fi
 
-if [ ! -z $DO_DRY_RUN  ]
+if [ -n "$DO_DRY_RUN" ]
 then
     echo "#${COL_YELLOW} ////////////////////////////////////////////////////////////${COL_RESET}"
     echo "#${COL_YELLOW} /// ${COL_RESET} This is only a dry run; nothing will be modified! ${COL_YELLOW}///${COL_RESET}"
