@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  *
@@ -41,6 +41,33 @@
 #if HAVE_JSON_GLIB
 #include <json-glib/json-glib.h>
 
+typedef struct RmUnpackedDirectory {
+    GQueue *files;
+} RmUnpackedDirectory;
+
+static RmUnpackedDirectory *rm_unpacked_directory_new(void) {
+    RmUnpackedDirectory *self = g_malloc0(sizeof(RmUnpackedDirectory));
+    self->files = g_queue_new();
+    return self;
+}
+
+static bool rm_unpacked_directory_has_next(RmUnpackedDirectory *self) {
+    return g_queue_get_length(self->files) > 0;
+}
+
+static RmFile *rm_unpacked_directory_next(RmUnpackedDirectory *self) {
+    return g_queue_pop_head(self->files);
+}
+
+static void rm_unpacked_directory_add(RmUnpackedDirectory *self, RmFile *file) {
+    g_queue_push_tail(self->files, file);
+}
+
+static void rm_unpacked_directory_free(RmUnpackedDirectory *self) {
+    g_queue_free(self->files);
+    g_free(self);
+}
+
 /////////////////////////////////////////////////
 //  POLLY THE PARROT REPEATS WHAT RMLINT SAID  //
 /////////////////////////////////////////////////
@@ -69,14 +96,48 @@ typedef struct RmParrot {
     /* true if the .json file came after a //
      * on the commandline. */
     bool is_prefd;
+
+    /* If true deliver each RmFile in a duplicate directory. */
+    bool unpack_directories;
+    RmUnpackedDirectory *unpacker;
+
+    /* If true, cluster up found RmFiles */
+    bool pack_directories;
+
+    /* map duplicate directories paths to the group of RmFiles
+     * they consist of (read from the "part_of_directory" type
+     * (required for the unpacking feature)
+     * */
+    RmTrie directory_trie;
 } RmParrot;
 
+static int rm_parrot_dir_trie_clear_children(_UNUSED RmTrie *self, RmNode *node, _UNUSED int level, _UNUSED void *user_data) {
+    GQueue *children = node->data;
+    if(children != NULL) {
+        g_queue_free(children);
+    }
+    return 0;
+}
+
 static void rm_parrot_close(RmParrot *polly) {
     if(polly->parser) {
         g_object_unref(polly->parser);
     }
 
     g_hash_table_unref(polly->disk_ids);
+
+    /* Free the GQeues in the trie */
+    rm_trie_iter(
+        &polly->directory_trie,
+        NULL,
+        true,
+        true,
+        rm_parrot_dir_trie_clear_children,
+        NULL
+    );
+
+    rm_trie_destroy(&polly->directory_trie);
+
     g_free(polly);
 }
 
@@ -88,10 +149,10 @@ static RmParrot *rm_parrot_open(RmSession *session, const char *json_path, bool
     polly->disk_ids = g_hash_table_new(NULL, NULL);
     polly->index = 1;
     polly->is_prefd = is_prefd;
+    rm_trie_init(&polly->directory_trie);
 
     for(GSList *iter = session->cfg->paths; iter; iter = iter->next) {
         RmPath *rmpath = iter->data;
-
         RmStat stat_buf;
         if(rm_sys_stat(rmpath->path, &stat_buf) != -1) {
             g_hash_table_add(polly->disk_ids, GUINT_TO_POINTER(stat_buf.st_dev));
@@ -99,24 +160,56 @@ static RmParrot *rm_parrot_open(RmSession *session, const char *json_path, bool
     }
 
     if(!json_parser_load_from_file(polly->parser, json_path, error)) {
-        goto failure;
+        return NULL;
     }
 
     JsonNode *root = json_parser_get_root(polly->parser);
     if(JSON_NODE_TYPE(root) != JSON_NODE_ARRAY) {
         g_set_error(error, RM_ERROR_QUARK, 0, _("No valid json cache (no array in /)"));
-        goto failure;
+        return NULL;
     }
 
     polly->root = json_node_get_array(root);
-    return polly;
 
-failure:
-    rm_parrot_close(polly);
-    return NULL;
+    JsonObject *object = json_array_get_object_element(polly->root, 0);
+    JsonNode *merge_directories_node = json_object_get_member(object, "merge_directories");
+
+    if(merge_directories_node != NULL) {
+        bool json_had_merge_dirs = json_node_get_boolean(merge_directories_node);
+
+        if(session->cfg->merge_directories != json_had_merge_dirs) {
+            if(json_had_merge_dirs) {
+                /* The .json file was created with the -D option
+                 * We need to unpack directories while running.
+                 * */
+                rm_log_info_line("»%s« was created with -D, but you're running without.", json_path);
+                rm_log_info_line("rmlint will unpack duplicate directories into individual files.");
+                rm_log_info_line("If you do not want this, pass -D to the next run.");
+                rm_log_info_line("NOTE: This feature is still considered EXPERIMENTAL!");
+                polly->unpack_directories = true;
+            } else {
+                rm_log_info_line("»%s« was created without -D, but you're running with.", json_path);
+                rm_log_info_line("rmlint will pack duplicate files into directories where applicable.");
+                rm_log_info_line("If you do not want this, omit -D from the next run.");
+                rm_log_info_line("NOTE: This feature is still considered EXPERIMENTAL!");
+                polly->pack_directories = true;
+            }
+        }
+    }
+
+    return polly;
 }
 
 static bool rm_parrot_has_next(RmParrot *polly) {
+    if(polly->unpacker != NULL) {
+        if(rm_unpacked_directory_has_next(polly->unpacker)) {
+            return true;
+        }
+
+        rm_unpacked_directory_free(polly->unpacker);
+        polly->unpacker = NULL;
+    }
+
     return (polly->index < json_array_get_length(polly->root));
 }
 
@@ -141,11 +234,6 @@ static RmFile *rm_parrot_try_next(RmParrot *polly) {
 
     path = json_node_get_string(path_node);
 
-    if(rm_trie_search_node(&polly->session->cfg->file_trie, path)) {
-        /* We have this node already */
-        return NULL;
-    }
-
     /* Check for the lint type */
     RmLintType type =
         rm_file_string_to_lint_type(json_object_get_string_member(object, "type"));
@@ -173,7 +261,8 @@ static RmFile *rm_parrot_try_next(RmParrot *polly) {
     /* Check if we're late and issue an warning */
     JsonNode *mtime_node = json_object_get_member(object, "mtime");
     if(mtime_node) {
-        gdouble stat_mtime = rm_sys_stat_mtime_float(stat_info);
+        /* Note: lstat_buf used here since for symlinks we want their mtime */
+        gdouble stat_mtime = rm_sys_stat_mtime_float(&lstat_buf);
         gdouble json_mtime = json_node_get_double(mtime_node);
 
         /* Allow them a rather large span to deviate to account for inaccuracies */
@@ -188,12 +277,27 @@ static RmFile *rm_parrot_try_next(RmParrot *polly) {
     file->is_original = json_object_get_boolean_member(object, "is_original");
     file->is_symlink = (lstat_buf.st_mode & S_IFLNK);
     file->digest = rm_digest_new(RM_DIGEST_EXT, 0);
-    file->free_digest = true;
 
-    // stat() reports directories as size zero.
-    // Fix this by actually using the size field in this example.
-    if (type == RM_LINT_TYPE_DUPE_DIR_CANDIDATE && stat_info->st_mode & S_IFDIR) {
-        file->actual_file_size = json_object_get_int_member(object, "size");
+    if(type != RM_LINT_TYPE_UNIQUE_FILE) {
+        file->free_digest = true;
+    } else {
+        file->free_digest = false;
+    }
+
+    if(type == RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
+        // stat() reports directories as size zero.
+        // Fix this by actually using the size field from the json node.
+        if(stat_info->st_mode & S_IFDIR) {
+            file->actual_file_size = json_object_get_int_member(object, "size");
+        }
+
+        file->n_children = (size_t)json_object_get_int_member(object, "n_children");
+    }
+
+    // If the file is a symbolic link and we remove it,
+    // we should not count the target size.
+    if(type == RM_LINT_TYPE_DUPE_CANDIDATE && file->is_symlink) {
+        file->actual_file_size = lstat_buf.st_size;
     }
 
     if(file->is_original) {
@@ -222,16 +326,95 @@ static RmFile *rm_parrot_try_next(RmParrot *polly) {
         g_assert(!file->hardlinks);
     }
 
+    if(file->lint_type == RM_LINT_TYPE_PART_OF_DIRECTORY) {
+        const char *parent_path = json_object_get_string_member(object, "parent_path");
+        GQueue *children = rm_trie_search(&polly->directory_trie, parent_path);
+        if(children == NULL) {
+            children = g_queue_new();
+            rm_trie_insert(&polly->directory_trie, parent_path, children);
+        }
+
+        g_queue_push_tail(children, file);
+    }
+
     return file;
 }
 
+static int rm_parrot_iter_dir_children(_UNUSED RmTrie *self, RmNode *node, _UNUSED int level, void *user_data) {
+    RmUnpackedDirectory *unpacker = user_data;
+
+    char buf[PATH_MAX] = {0};
+    rm_trie_build_path_unlocked(node, buf, PATH_MAX);
+
+    GQueue *children = node->data;
+    if(children == NULL) {
+        return 0;
+    }
+
+    for(GList *iter = children->head; iter; iter = iter->next) {
+        // Mask that RM_LINT_TYPE_PART_OF_DIRECTORY as normal duplicate.
+        RmFile *child_file = rm_file_copy(iter->data);
+        child_file->lint_type = RM_LINT_TYPE_DUPE_CANDIDATE;
+        rm_unpacked_directory_add(unpacker, child_file);
+    }
+
+    // Make sure to clear the queue. This is important in case a more top-level
+    // directory contains this directory. Then we would output the same files
+    // twice.
+    g_queue_clear(children);
+    return 0;
+}
+
 static RmFile *rm_parrot_next(RmParrot *polly) {
-    /* Skip NULL entries */
-    while(rm_parrot_has_next(polly)) {
-        RmFile *file = NULL;
-        if((file = rm_parrot_try_next(polly))) {
+    RmFile *file = NULL;
+
+    /* If we have a directory to unpack, let's do that first */
+    if(polly->unpacker != NULL) {
+        file = rm_unpacked_directory_next(polly->unpacker);
+        if(file != NULL) {
             return file;
         }
+
+        rm_unpacked_directory_free(polly->unpacker);
+        polly->unpacker = NULL;
+    }
+
+    /* Skip NULL entries */
+    while(rm_parrot_has_next(polly)) {
+        file = rm_parrot_try_next(polly);
+        if(file == NULL) {
+            /* try again */
+            continue;
+        }
+
+        if(polly->unpack_directories &&
+           file->lint_type == RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
+            RM_DEFINE_PATH(file);
+            rm_log_debug_line("unpacking: %s", file_path);
+
+            /* Accumulate all files in that directory into a group.
+             * This group will serve as source for the next iterations.
+             */
+            polly->unpacker = rm_unpacked_directory_new();
+            rm_trie_iter(
+                &polly->directory_trie,
+                rm_trie_search_node(&polly->directory_trie, file_path),
+                true,
+                true,
+                rm_parrot_iter_dir_children,
+                polly->unpacker
+            );
+
+            /* we can't get rid of the actual directory now */
+            rm_file_destroy(file);
+
+            /* call self, which will now read from the unpacker;
+             * current duplicate directory is not returned to caller.
+             * */
+            return rm_parrot_next(polly);
+        }
+
+        return file;
     }
 
     return NULL;
@@ -259,13 +442,19 @@ static bool rm_parrot_check_size(RmCfg *cfg, RmFile *file) {
         return true;
     }
 
-    return ((cfg->minsize == (RmOff)-1 || cfg->minsize <= file->actual_file_size) &&
-            (cfg->maxsize == (RmOff)-1 || file->actual_file_size <= cfg->maxsize));
+    if((cfg->minsize == (RmOff)-1 || cfg->minsize <= file->actual_file_size)
+    && (cfg->maxsize == (RmOff)-1 || file->actual_file_size <= cfg->maxsize)) {
+        return true;
+    }
+
+    FAIL_MSG("nope: bad size");
+    return false;
 }
 
 static bool rm_parrot_check_hidden(RmCfg *cfg, _UNUSED RmFile *file,
                                    const char *file_path) {
-    if(!cfg->ignore_hidden) {
+    if(cfg->ignore_hidden == false && cfg->partial_hidden == false) {
+        // no need to check.
         return true;
     }
 
@@ -317,7 +506,6 @@ static bool rm_parrot_check_path(RmParrot *polly, RmFile *file, const char *file
      * into a RmTrie and use it to find the longest prefix easily.
      */
 
-    /* iterate through cmdline paths */
     for(GSList *iter = cfg->paths; iter; iter = iter->next) {
         RmPath *rmpath = iter->data;
         size_t path_len = strlen(rmpath->path);
@@ -359,6 +547,8 @@ static bool rm_parrot_check_types(RmCfg *cfg, RmFile *file) {
         return cfg->find_badids;
     case RM_LINT_TYPE_UNIQUE_FILE:
         return cfg->write_unfinished;
+    case RM_LINT_TYPE_PART_OF_DIRECTORY:
+        return true;
     case RM_LINT_TYPE_UNKNOWN:
     default:
         FAIL_MSG("nope: invalid lint type.");
@@ -372,19 +562,64 @@ static bool rm_parrot_check_types(RmCfg *cfg, RmFile *file) {
 
 /* RmHRFunc to remove file unless it has a twin in group */
 static int rm_parrot_fix_match_opts(RmFile *file, GQueue *group) {
-    int remove = 1;
-
     for(GList *iter = group->head; iter; iter = iter->next) {
         if(file != iter->data && rm_file_cmp(file, iter->data) == 0) {
-            remove = 0;
-            break;
+            return 0;
         }
     }
 
-    if(remove) {
-        rm_file_destroy(file);
+    return 1;
+}
+
+static int rm_parrot_sort_by_path(gconstpointer a, gconstpointer b, _UNUSED gpointer data) {
+    const RmFile *file_a = a;
+    const RmFile *file_b = b;
+
+    RM_DEFINE_PATH(file_a);
+    RM_DEFINE_PATH(file_b);
+    return strcmp(file_a_path, file_b_path);
+}
+
+static void rm_parrot_fix_duplicate_entries(RmParrotCage *cage, GQueue *group) {
+    /* This quirk can happen when we have a duplicate directory that
+     * was unpacked. If that dir has other duplicates inside them
+     * it can happen that we have a "part_of_directory" type that
+     * was promoted to a "dupe_canidate" before.
+     *
+     * This also serves as safety-net for cases when the json files
+     * contain a path several times.
+     */
+
+    g_queue_sort(
+        group,
+        (GCompareDataFunc)rm_parrot_sort_by_path,
+        NULL
+    );
+
+    char *last_path = NULL;
+
+    for(GList *iter = group->head; iter; iter = iter->next) {
+        RmFile *file = iter->data;
+        if(file->lint_type == RM_LINT_TYPE_PART_OF_DIRECTORY) {
+            /* Those are not visible in the output, allow them. */
+            continue;
+        }
+
+        RM_DEFINE_PATH(file);
+
+        if(last_path && g_strcmp0(file_path, last_path) == 0) {
+            // remove node.
+            GList *old_iter = iter;
+            iter = iter->prev;
+            g_queue_delete_link(cage->groups, old_iter);
+            rm_file_destroy(file);
+        }
+
+        g_free(last_path);
+        last_path = g_strdup(file_path);
     }
-    return remove;
+
+    g_free(last_path);
 }
 
 static void rm_parrot_fix_must_match_tagged(RmParrotCage *cage, GQueue *group) {
@@ -397,6 +632,11 @@ static void rm_parrot_fix_must_match_tagged(RmParrotCage *cage, GQueue *group) {
 
     for(GList *iter = group->head; iter; iter = iter->next) {
         RmFile *file = iter->data;
+        if(file->lint_type != RM_LINT_TYPE_DUPE_CANDIDATE &&
+           file->lint_type != RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
+            // -k and -m only applies to dupes.
+            return;
+        }
 
         has_prefd |= file->is_prefd;
         has_non_prefd |= !file->is_prefd;
@@ -407,7 +647,6 @@ static void rm_parrot_fix_must_match_tagged(RmParrotCage *cage, GQueue *group) {
 
     if((!has_prefd && cfg->must_match_tagged) ||
        (!has_non_prefd && cfg->must_match_untagged)) {
-        g_queue_foreach(group, (GFunc)rm_file_destroy, NULL);
         g_queue_clear(group);
     }
 }
@@ -415,23 +654,36 @@ static void rm_parrot_fix_must_match_tagged(RmParrotCage *cage, GQueue *group) {
 static void rm_parrot_update_stats(RmParrotCage *cage, RmFile *file) {
     RmSession *session = cage->session;
 
+    if(file->lint_type == RM_LINT_TYPE_PART_OF_DIRECTORY) {
+        return;
+    }
+
     session->total_files += 1;
+    if(file->lint_type == RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
+        session->total_files += file->n_children;
+    }
+
     if(file->lint_type == RM_LINT_TYPE_DUPE_CANDIDATE ||
        file->lint_type == RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
         session->dup_group_counter += file->is_original;
         if(!file->is_original) {
             session->dup_counter += 1;
 
+            if(file->lint_type == RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
+                session->dup_counter += file->n_children;
+            }
+
             if(!RM_FILE_IS_HARDLINK(file)) {
                 session->total_lint_size += file->actual_file_size;
             }
         }
+
     } else {
         session->other_lint_cnt += 1;
     }
 }
 
-static void rm_parrot_write_group(RmParrotCage *cage, GQueue *group) {
+static void rm_parrot_cage_write_group(RmParrotCage *cage, GQueue *group, bool pack_directories) {
     RmCfg *cfg = cage->session->cfg;
 
     if(cfg->filter_mtime) {
@@ -442,37 +694,60 @@ static void rm_parrot_write_group(RmParrotCage *cage, GQueue *group) {
         }
 
         if(older == group->length) {
-            g_queue_foreach(group, (GFunc)rm_file_destroy, NULL);
             g_queue_clear(group);
             return;
         }
     }
 
-    if(cfg->match_with_extension || cfg->match_without_extension || cfg->match_basename ||
-       cfg->unmatched_basenames) {
+    if(cfg->match_with_extension
+    || cfg->match_without_extension
+    || cfg->match_basename
+    || cfg->unmatched_basenames) {
         /* This is probably a sucky way to do it, due to n^2,
          * but I doubt that will make a large performance difference.
          */
-        rm_util_queue_foreach_remove(group, (RmRFunc)rm_parrot_fix_match_opts, group);
+        rm_util_queue_foreach_remove(
+            group,
+            (RmRFunc)rm_parrot_fix_match_opts,
+            group
+        );
     }
     rm_parrot_fix_must_match_tagged(cage, group);
+    rm_parrot_fix_duplicate_entries(cage, group);
 
-    g_queue_sort(group, (GCompareDataFunc)rm_shred_cmp_orig_criteria, cage->session);
+    g_queue_sort(
+        group,
+        (GCompareDataFunc)rm_shred_cmp_orig_criteria,
+        cage->session
+    );
 
     for(GList *iter = group->head; iter; iter = iter->next) {
         RmFile *file = iter->data;
 
-        if(file == group->head->data || (cfg->keep_all_tagged && file->is_prefd) ||
-           (cfg->keep_all_untagged && !file->is_prefd)) {
+        if(file == group->head->data
+        || (cfg->keep_all_tagged && file->is_prefd)
+        || (cfg->keep_all_untagged && !file->is_prefd)) {
             file->is_original = true;
         } else {
             file->is_original = false;
         }
 
-        RM_DEFINE_PATH(file);
+        /* Other lint never should bother for is_original,
+         * since this definition doesn't make sense there.
+         * */
+        if(file->lint_type != RM_LINT_TYPE_DUPE_CANDIDATE
+        && file->lint_type != RM_LINT_TYPE_DUPE_DIR_CANDIDATE) {
+            file->is_original = false;
+        }
 
         rm_parrot_update_stats(cage, file);
-        rm_fmt_write(file, cage->session->formats, group->length);
+        file->twin_count = group->length;
+
+        if(pack_directories && file->lint_type == RM_LINT_TYPE_DUPE_CANDIDATE) {
+            rm_tm_feed(cage->tree_merger, file);
+        } else {
+            rm_fmt_write(file, cage->session->formats);
+        }
     }
 }
 
@@ -480,13 +755,18 @@ static void rm_parrot_write_group(RmParrotCage *cage, GQueue *group) {
 //  ENTRY POINT TO TRIGGER THE PARROT  //
 /////////////////////////////////////////
 
-static void rm_parrot_cage_push_group(RmParrotCage *cage, GQueue **group_ref,
+static void rm_parrot_cage_push_to_group(RmParrotCage *cage, GQueue **group_ref,
                                       bool is_last) {
     GQueue *group = *group_ref;
-    if(group->length > 1) {
+
+    // NOTE: We allow groups with only one file in it.
+    // Those can happen when we unpack directories.
+    // If there's really just one file in the group
+    // it is kicked our later in the process.
+    if(group->length > 0) {
         g_queue_push_tail(cage->groups, group);
     } else {
-        g_queue_free_full(group, (GDestroyNotify)rm_file_destroy);
+        g_queue_free(group);
     }
 
     if(!is_last) {
@@ -508,6 +788,7 @@ bool rm_parrot_cage_load(RmParrotCage *cage, const char *json_path, bool is_pref
 
     RmCfg *cfg = cage->session->cfg;
     GQueue *group = g_queue_new();
+    GQueue *part_of_directory_entries = g_queue_new();
     RmDigest *last_digest = NULL;
 
     /* group of files; first group is "other lint" */
@@ -521,13 +802,29 @@ bool rm_parrot_cage_load(RmParrotCage *cage, const char *json_path, bool is_pref
         rm_log_debug("Checking `%s`: ", file_path);
 
         /* Check --size, --perms, --hidden */
-        if(!(rm_parrot_check_depth(cfg, file) && rm_parrot_check_size(cfg, file) &&
-             rm_parrot_check_hidden(cfg, file, file_path) &&
-             rm_parrot_check_permissions(cfg, file, file_path) &&
-             rm_parrot_check_types(cfg, file) && rm_parrot_check_crossdev(polly, file) &&
-             rm_parrot_check_path(polly, file, file_path))) {
-            rm_file_destroy(file);
+        if(!(
+               rm_parrot_check_depth(cfg, file)
+            && rm_parrot_check_size(cfg, file)
+            && rm_parrot_check_hidden(cfg, file, file_path)
+            && rm_parrot_check_permissions(cfg, file, file_path)
+            && rm_parrot_check_types(cfg, file)
+            && rm_parrot_check_crossdev(polly, file)
+            && rm_parrot_check_path(polly, file, file_path)
+        )) {
             rm_log_debug("[nope]\n");
+            if(file->lint_type != RM_LINT_TYPE_PART_OF_DIRECTORY) {
+                rm_file_destroy(file);
+            }
+
+            continue;
+        }
+
+        /* Special case for part of dirs:
+         * Accumuluate those in a single group that come in front of the other groups.
+         * */
+        if(file->lint_type == RM_LINT_TYPE_PART_OF_DIRECTORY) {
+            rm_log_debug("[part of directory]\n");
+            g_queue_push_tail(part_of_directory_entries, file);
             continue;
         }
 
@@ -537,10 +834,10 @@ bool rm_parrot_cage_load(RmParrotCage *cage, const char *json_path, bool is_pref
 
         rm_log_debug("[okay]\n");
 
-        if(!rm_digest_equal(file->digest, last_digest)) {
+        if(file->digest != NULL && !rm_digest_equal(file->digest, last_digest)) {
             rm_digest_free(last_digest);
             last_digest = rm_digest_copy(file->digest);
-            rm_parrot_cage_push_group(cage, &group, false);
+            rm_parrot_cage_push_to_group(cage, &group, false);
         }
 
         g_queue_push_tail(group, file);
@@ -550,14 +847,23 @@ bool rm_parrot_cage_load(RmParrotCage *cage, const char *json_path, bool is_pref
         rm_digest_free(last_digest);
     }
 
-    rm_parrot_cage_push_group(cage, &group, true);
-    rm_parrot_close(polly);
+    rm_parrot_cage_push_to_group(cage, &group, true);
+    g_queue_push_tail(cage->parrots, polly);
+
+    if(part_of_directory_entries->length > 1) {
+        g_queue_push_head(cage->groups, part_of_directory_entries);
+    } else {
+        g_queue_free_full(part_of_directory_entries, (GDestroyNotify)rm_file_destroy);
+    }
+
     return true;
 }
 
 void rm_parrot_cage_open(RmParrotCage *cage, RmSession *session) {
     cage->session = session;
     cage->groups = g_queue_new();
+    cage->parrots = g_queue_new();
+    cage->tree_merger = NULL;
 }
 
 static void rm_parrot_merge_identical_groups(RmParrotCage *cage) {
@@ -567,16 +873,22 @@ static void rm_parrot_merge_identical_groups(RmParrotCage *cage) {
     for(GList *iter = cage->groups->head; iter; iter = iter->next) {
         GQueue *group = iter->data;
         RmFile *head_file = group->head->data;
+
         GQueue *existing_group = g_hash_table_lookup(digest_to_group, head_file->digest);
 
         if(existing_group != NULL) {
-            // Merge the two groups and get rid of the old one.
-            rm_util_queue_push_tail_queue(existing_group, group);
-            g_queue_free(group);
-            GList *old_iter = iter;
-            iter = iter->prev;
-            g_queue_delete_link(cage->groups, old_iter);
-            continue;
+            RmFile *existing_head_file = existing_group->head->data;
+
+            /* Merge only groups with the same type */
+            if(existing_head_file->lint_type == head_file->lint_type) {
+                /* Merge the two groups and get rid of the old one. */
+                rm_util_queue_push_tail_queue(existing_group, group);
+                g_queue_free(group);
+                GList *old_iter = iter;
+                iter = iter->prev;
+                g_queue_delete_link(cage->groups, old_iter);
+                continue;
+            }
         }
 
         // No such group, but remember it.
@@ -586,19 +898,53 @@ static void rm_parrot_merge_identical_groups(RmParrotCage *cage) {
     g_hash_table_unref(digest_to_group);
 }
 
-void rm_parrot_cage_close(RmParrotCage *cage) {
+void rm_parrot_cage_output_treemerge_results(RmFile *file, gpointer data) {
+    RmParrotCage *cage = data;
+    g_assert(cage);
+
+    rm_fmt_write(file, cage->session->formats);
+}
+
+void rm_parrot_cage_flush(RmParrotCage *cage) {
     rm_parrot_merge_identical_groups(cage);
 
+    bool pack_directories = false;
+
+    /* Check if any of the .json files were created with -D.
+     * If so, we need to merge them up again using treemerge.c
+     */
+    for(GList *iter = cage->parrots->head; iter; iter = iter->next) {
+        RmParrot *parrot = iter->data;
+        if(parrot->pack_directories) {
+            pack_directories = true;
+            break;
+        }
+    }
+
+    if(pack_directories) {
+        cage->tree_merger = rm_tm_new(cage->session);
+        rm_tm_set_callback(cage->tree_merger, (RmTreeMergeOutputFunc)rm_parrot_cage_output_treemerge_results, cage); 
+    }
+
     for(GList *iter = cage->groups->head; iter; iter = iter->next) {
         GQueue *group = iter->data;
         if(group->length > 1) {
-            rm_parrot_write_group(cage, group);
-        } else {
-            g_queue_free_full(group, (GDestroyNotify)rm_file_destroy);
+            rm_parrot_cage_write_group(cage, group, pack_directories);
         }
     }
 
+    if(pack_directories && cage->tree_merger) {
+        rm_tm_finish(cage->tree_merger);
+    }
+}
+
+void rm_parrot_cage_close(RmParrotCage *cage) {
     g_queue_free_full(cage->groups, (GDestroyNotify)g_queue_free);
+    g_queue_free_full(cage->parrots, (GDestroyNotify)rm_parrot_close);
+
+    if(cage->tree_merger) {
+        rm_tm_destroy(cage->tree_merger);
+    }
 }
 
 #else
@@ -613,6 +959,9 @@ void rm_parrot_cage_open(_UNUSED RmParrotCage *cage, _UNUSED RmSession *session)
     rm_log_error_line(_("Please recompile `rmlint` with it installed."));
 }
 
+void rm_parrot_cage_flush(_UNUSED RmParrotCage *cage) {
+}
+
 void rm_parrot_cage_close(_UNUSED RmParrotCage *cage) {
 }
 
