@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  *
@@ -47,6 +47,8 @@
 typedef struct RmParrotCage {
     RmSession *session;
     GQueue *groups;
+    GQueue *parrots;
+    RmTreeMerger *tree_merger;
 } RmParrotCage;
 
 /**
@@ -68,4 +70,9 @@ bool rm_parrot_cage_load(RmParrotCage *cage, const char *json_path, bool is_pref
  */
 void rm_parrot_cage_close(RmParrotCage *cage);
 
+/**
+ * @brief Flush the cage's contents to the output module.
+ * */
+void rm_parrot_cage_flush(RmParrotCage *cage);
+
 #endif /* end of include guard */
