@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  *
@@ -357,6 +357,9 @@ typedef struct RmShredGroup {
     /* number of file clusters */
     gsize n_clusters;
 
+    /* number of file clusters that don't have external checksums */
+    gsize n_unhashed_clusters;
+
     /* number of distinct inodes */
     gsize n_inodes;
 
@@ -572,7 +575,7 @@ static gulong rm_shred_group_potential_file_count(RmShredGroup *group) {
     }
 }
 
-/* Governer to limit memory usage by limiting how many RmShredGroups can be
+/* Governor to limit memory usage by limiting how many RmShredGroups can be
  * active at any one time
  * NOTE: group_lock must be held before calling rm_shred_check_paranoid_mem_alloc
  */
@@ -684,9 +687,25 @@ static void rm_shred_counter_factory(RmCounterBuffer *buffer, RmShredTag *tag) {
     g_slice_free(RmCounterBuffer, buffer);
 }
 
-static void rm_shred_write_cksum_to_xattr(const RmSession *session, RmFile *file) {
-    if(session->cfg->write_cksum_to_xattr) {
-        if(!file->ext_cksum) {
+static void rm_shred_write_group_to_xattr(const RmSession *session, GQueue *group) {
+    if(session->cfg->write_cksum_to_xattr == false) {
+        /* feature is not requested, bail out */
+        return;
+    }
+
+    if(session->cfg->write_unfinished == false && g_queue_get_length(group) <= 1) {
+        /* Do not write solitary files unless requested */
+        return;
+    }
+
+    if(session->cfg->clamp_is_used) {
+        /* Not writing checksums because we're not reading the whole file. */
+        return;
+    }
+
+    for(GList *iter = group->head; iter; iter = iter->next) {
+        RmFile *file = iter->data;
+        if(file->ext_cksum == NULL && file->digest != NULL) {
             rm_xattr_write_hash(file, (RmSession *)session);
         }
     }
@@ -830,11 +849,18 @@ static void rm_shred_group_finalise(RmShredGroup *self) {
  * Assume group already protected by group_lock.
  * */
 static void rm_shred_group_update_status(RmShredGroup *group) {
-    if(group->status == RM_SHRED_GROUP_DORMANT && rm_shred_group_qualifies(group) &&
-       group->hash_offset < group->file_size &&
-       (group->n_clusters > 1 ||
-        (group->n_inodes == 1 && group->session->cfg->merge_directories))) {
-        /* group can go active */
+    if(group->status != RM_SHRED_GROUP_DORMANT || !rm_shred_group_qualifies(group) ||
+       group->hash_offset == group->file_size) {
+        return;
+    }
+
+    if(group->n_clusters > 1) {
+        // we have potential match candidates; start hashing
+        group->status = RM_SHRED_GROUP_START_HASHING;
+    } else if(group->n_inodes == 1 && group->n_unhashed_clusters > 0 &&
+              group->session->cfg->merge_directories) {
+        /* special case of hardlinked files that still need hashing to help identify
+         * matching directories */
         group->status = RM_SHRED_GROUP_START_HASHING;
     }
 }
@@ -896,6 +922,9 @@ static RmFile *rm_shred_group_push_file(RmShredGroup *shred_group, RmFile *file,
         shred_group->n_npref += rm_file_n_nprefd(file);
         shred_group->n_new += rm_file_n_new(file);
         shred_group->n_clusters++;
+        if(file->ext_cksum == NULL) {
+            shred_group->n_unhashed_clusters++;
+        }
         shred_group->n_inodes += RM_FILE_INODE_COUNT(file);
 
         g_assert(file->hash_offset == shred_group->hash_offset);
@@ -1015,21 +1044,19 @@ static RmFile *rm_shred_sift(RmFile *file) {
     return result;
 }
 
-/* Hasher callback when file increment hashing is completed.
- * */
-static void rm_shred_hash_callback(_UNUSED RmHasher *hasher, RmDigest *digest,
-                                   RmShredTag *tag, RmFile *file) {
+/* Hasher callback when file increment hashing is completed. */
+static void rm_shred_hash_callback(
+    _UNUSED RmHasher *hasher,
+    RmDigest *digest,
+    _UNUSED RmShredTag *tag,
+    RmFile *file
+) {
     if(!file->digest) {
         file->digest = digest;
     }
     g_assert(file->digest == digest);
     g_assert(file->hash_offset == file->shred_group->next_offset);
 
-    if(file->status != RM_FILE_STATE_IGNORE) {
-        /* remember that checksum */
-        rm_shred_write_cksum_to_xattr(tag->session, file);
-    }
-
     if(file->shredder_waiting) {
         /* MDS scheduler is waiting for result */
         rm_signal_done(file->signal);
@@ -1214,12 +1241,12 @@ int rm_shred_cmp_orig_criteria(RmFile *a, RmFile *b, RmSession *session) {
               (cfg->keep_all_untagged || cfg->must_match_untagged)) {
         return (a->is_prefd - b->is_prefd);
     } else {
-        int comparasion = rm_pp_cmp_orig_criteria(a, b, session);
-        if(comparasion == 0) {
+        int comparison = rm_pp_cmp_orig_criteria(a, b, session);
+        if(comparison == 0) {
             return b->is_original - a->is_original;
         }
 
-        return comparasion;
+        return comparison;
     }
 }
 
@@ -1277,13 +1304,13 @@ static gboolean rm_shred_has_duplicates(GQueue *group) {
             return TRUE;
         }
     }
+
     return FALSE;
 }
 
 void rm_shred_forward_to_output(RmSession *session, GQueue *group) {
     g_assert(session);
     g_assert(group);
-    g_assert(group->head);
 
 #if _RM_SHRED_DEBUG
     RmFile *head = group->head->data;
@@ -1295,7 +1322,8 @@ void rm_shred_forward_to_output(RmSession *session, GQueue *group) {
     if(rm_shred_has_duplicates(group)) {
         for(GList *iter = group->head; iter; iter = iter->next) {
             RmFile *file = iter->data;
-            rm_fmt_write(file, session->formats, group->length);
+            file->twin_count = group->length;
+            rm_fmt_write(file, session->formats);
         }
     }
 }
@@ -1485,6 +1513,8 @@ static void rm_shred_group_postprocess(RmShredGroup *group, RmShredTag *tag) {
         rm_shred_forward_to_output(tag->session, group->held_files);
     }
 
+    rm_shred_write_group_to_xattr(tag->session, group->held_files);
+
     if(group->status == RM_SHRED_GROUP_FINISHING) {
         group->status = RM_SHRED_GROUP_FINISHED;
     }
@@ -1638,7 +1668,7 @@ static gint rm_shred_process_file(RmFile *file, RmSession *session) {
              (!cfg->shred_never_wait && rm_mds_device_is_rotational(file->disk) &&
               bytes_to_read < SHRED_TOO_MANY_BYTES_TO_WAIT));
 
-        gsize bytes_read = 0;
+        guint64 bytes_read = 0;
         RmHasherTask *task = rm_hasher_task_new(tag->hasher, file->digest, file);
         if(!rm_hasher_task_hash(task, file_path, file->hash_offset, bytes_to_read,
                                 file->is_symlink, &bytes_read)) {
@@ -1693,6 +1723,14 @@ static gint rm_shred_process_file(RmFile *file, RmSession *session) {
     return result;
 }
 
+/* called when treemerge.c found something interesting */
+void rm_shred_output_tm_results(RmFile *file, gpointer data) {
+    g_assert(data);
+
+    RmSession *session = data;
+    rm_fmt_write(file, session->formats);
+}
+
 void rm_shred_run(RmSession *session) {
     g_assert(session);
     g_assert(session->tables);
