@@ -16,8 +16,8 @@
  *
  * Authors:
  *
- *  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
- *  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+ *  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+ *  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
  *
  * Hosted on http://github.com/sahib/rmlint
  *
@@ -34,7 +34,7 @@
  * I.e. find duplicate directories.
  *
  * All API here is defined on the opaque RmTreeMerger structure.
- * Files are feeded and the finished directories are wrapped
+ * Files are fed and the finished directories are wrapped
  * as RmFiles and written to the output module.
  */
 
@@ -44,11 +44,22 @@ typedef struct RmTreeMerger RmTreeMerger;
 
 /* RmTreeMerger is part of RmSession, therefore prototype it here */
 struct RmSession;
+typedef struct RmSession RmSession;
+
+/**
+ * @brief callback function called to output a single directory.
+ */
+typedef gint (*RmTreeMergeOutputFunc)(RmFile *result, gpointer user_data);
 
 /**
  * @brief Allocate a new RmTreeMerger structure.
  */
-RmTreeMerger *rm_tm_new(struct RmSession *session);
+RmTreeMerger *rm_tm_new(RmSession *session);
+
+/**
+ * @brief Set the output callback
+ */
+void rm_tm_set_callback(RmTreeMerger *self, RmTreeMergeOutputFunc callback, gpointer callback_data);
 
 /**
  * @brief Add an RmFile to the pool of (to be) investigated files.
@@ -56,7 +67,7 @@ RmTreeMerger *rm_tm_new(struct RmSession *session);
 void rm_tm_feed(RmTreeMerger *self, RmFile *file);
 
 /**
- * @brief Find duplicate directories through all feeded RmFiles.
+ * @brief Find duplicate directories through all fed RmFiles.
  */
 void rm_tm_finish(RmTreeMerger *self);
 
@@ -65,4 +76,12 @@ void rm_tm_finish(RmTreeMerger *self);
  */
 void rm_tm_destroy(RmTreeMerger *self);
 
+/**
+ * A duplicate directory.
+ */
+struct RmDirectory;
+typedef struct RmDirectory RmDirectory;
+
+const char *rm_directory_get_dirname(RmDirectory *self);
+
 #endif /* RM_TREEMERGE_INCLUDE*/
