@@ -86,7 +86,7 @@
 #define RM_MOUNTTABLE_IS_USABLE (HAVE_BLKID && HAVE_GIO_UNIX)
 
 ////////////////////////////////////
-//       GENERAL UTILITES         //
+//       GENERAL UTILITIES         //
 ////////////////////////////////////
 
 char *rm_util_strsub(const char *string, const char *subs, const char *with) {
@@ -296,7 +296,7 @@ bool rm_util_is_nonstripped(_UNUSED const char *path, _UNUSED RmStat *statp) {
 #if HAVE_LIBELF
     g_return_val_if_fail(path, false);
 
-    if(statp && (statp->st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) == 0) {
+    if(!S_ISREG(statp->st_mode) || (statp->st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) == 0) {
         return false;
     }
 
@@ -588,7 +588,7 @@ static bool fs_supports_reflinks(char *fstype, char *mountpoint) {
     }
     if(strcmp(fstype, "xfs")==0) {
         /* xfs *might* support reflinks...*/
-        char *cmd = g_strdup_printf("xfs_info '%s' | grep -q 'reflink=1'", mountpoint);
+        char *cmd = g_strdup_printf("xfs_info '%s' 2>/dev/null | grep -q 'reflink=1'", mountpoint);
         int res = system(cmd);
         g_free(cmd);
         return(res==0);
@@ -629,7 +629,7 @@ static RmMountEntries *rm_mount_list_open(RmMountTable *table) {
             /* fsname as show by `mount` */
             const char *name;
 
-            /* Wether to warn about the exclusion on this */
+            /* Whether to warn about the exclusion on this */
             bool unusual;
         } evilfs_types[] = {{"bindfs", 1},
                             {"nullfs", 1},
@@ -889,7 +889,7 @@ dev_t rm_mounts_get_disk_id(RmMountTable *self, _UNUSED dev_t dev,
     }
 
     /* probably a btrfs subvolume which is not a mountpoint;
-     * walk up tree until we get to a recognisable partition
+     * walk up tree until we get to a recognizable partition
      * */
     char *prev = g_strdup(path);
     while(TRUE) {
@@ -978,13 +978,11 @@ bool rm_mounts_can_reflink(RmMountTable *self, dev_t source, dev_t dest) {
 }
 
 /////////////////////////////////
-//    FIEMAP IMPLEMENATION     //
+//    FIEMAP IMPLEMENTATION     //
 /////////////////////////////////
 
 #if HAVE_FIEMAP
 
-#define _RM_OFFSET_DEBUG 0
-
 /* Return fiemap structure containing n_extents for file descriptor fd.
  * Return NULL if errors encountered.
  * Needs to be freed with g_free if not NULL.
@@ -1013,19 +1011,20 @@ static struct fiemap *rm_offset_get_fiemap(int fd, const int n_extents,
     return fm;
 }
 
-/* Return physical (disk) offset of the beginning of the file extent containing the
+/* Return physical (disk) offset of the beginning of the file extent found after the
  * specified logical file_offset.
- * If a pointer to file_offset_next is provided then read fiemap extents until
- * the next non-contiguous extent (fragment) is encountered and writes the corresponding
- * file offset to &file_offset_next.
+ * If a pointer to file_offset_next is provided then read fiemap extents until the next
+ * non-contiguous extent (fragment) is encountered and writes the corresponding file
+ * offset to *file_offset_next.
+ * The logical offset of the found extent is written to *logical_offset.
  * */
-RmOff rm_offset_get_from_fd(int fd, RmOff file_offset, RmOff *file_offset_next, bool *is_last) {
+RmOff rm_offset_get_from_fd(int fd, RmOff file_offset, RmOff *file_offset_next, RmOff *logical_offset, bool *is_last) {
     RmOff result = 0;
     bool done = FALSE;
     bool first = TRUE;
+    struct fiemap_extent fm_last;
 
-    /* used for detecting contiguous extents */
-    unsigned long expected = 0;
+    memset(&fm_last, 0, sizeof(fm_last));
 
     fsync(fd);
 
@@ -1041,10 +1040,10 @@ RmOff rm_offset_get_from_fd(int fd, RmOff file_offset, RmOff *file_offset_next,
             break;
         }
 
-        if (fm->fm_mapped_extents == 0) {
-#if _RM_OFFSET_DEBUG
-            rm_log_info_line(_("rm_offset_get_fiemap: got no extents for %d"), fd);
-#endif
+        if(fm->fm_mapped_extents == 0) {
+            if(file_offset != 0) {
+                rm_log_warning_line(_("rm_offset_get_fiemap: got no extents for %d at offset %" G_GUINT64_FORMAT), fd, file_offset);
+            }
             done = TRUE;
         } else {
 
@@ -1054,20 +1053,21 @@ RmOff rm_offset_get_from_fd(int fd, RmOff file_offset, RmOff *file_offset_next,
             if (first) {
                 /* remember disk location of start of data */
                 result = fm_ext.fe_physical;
-                first=FALSE;
+                if(logical_offset != NULL) {
+                    *logical_offset = fm_ext.fe_logical;
+                }
+                first = FALSE;
             } else {
                 /* check if subsequent extents are contiguous */
-                if(fm_ext.fe_physical != expected)  {
+                unsigned long expected_dense = fm_last.fe_physical + fm_last.fe_length;
+                unsigned long expected = fm_last.fe_physical + fm_ext.fe_logical - fm_last.fe_logical;
+                if(fm_ext.fe_physical != expected || fm_ext.fe_physical != expected_dense) {
                     /* current extent is not contiguous with previous, so we can stop */
-                    done = TRUE;
+                    g_free(fm);
+                    break;
                 }
             }
 
-            if (!done && file_offset_next != NULL) {
-                /* update logical offset of next fragment */
-                *file_offset_next = fm_ext.fe_logical + fm_ext.fe_length;
-            }
-
             if(fm_ext.fe_flags & FIEMAP_EXTENT_LAST) {
                 done = TRUE;
 
@@ -1082,8 +1082,8 @@ RmOff rm_offset_get_from_fd(int fd, RmOff file_offset, RmOff *file_offset_next,
             }
 
             /* move offsets in preparation for reading next extent */
-            file_offset += fm_ext.fe_length;
-            expected = fm_ext.fe_physical + fm_ext.fe_length;
+            file_offset = fm_ext.fe_logical + fm_ext.fe_length;
+            fm_last = fm_ext;
         }
 
         g_free(fm);
@@ -1104,15 +1104,15 @@ RmOff rm_offset_get_from_path(const char *path, RmOff file_offset,
         rm_log_info("Error opening %s in rm_offset_get_from_path\n", path);
         return 0;
     }
-    RmOff result = rm_offset_get_from_fd(fd, file_offset, file_offset_next, NULL);
+    RmOff result = rm_offset_get_from_fd(fd, file_offset, file_offset_next, NULL, NULL);
     rm_sys_close(fd);
     return result;
 }
 
 #else /* Probably FreeBSD */
 
-RmOff rm_offset_get_from_fd(_UNUSED int fd, _UNUSED RmOff file_offset,
-                            _UNUSED RmOff *file_offset_next, _UNUSED bool *is_last) {
+RmOff rm_offset_get_from_fd(_UNUSED int fd, _UNUSED RmOff file_offset, _UNUSED RmOff *file_offset_next,
+                            _UNUSED RmOff *logical_offset, _UNUSED bool *is_last) {
     return 0;
 }
 
@@ -1249,28 +1249,31 @@ RmLinkType rm_util_link_type(char *path1, char *path2) {
 
     bool is_last_1 = false;
     bool is_last_2 = false;
-    bool at_least_one_checked = false;
 
     while(!rm_session_was_aborted()) {
-        RmOff logical_next_1 = 0;
-        RmOff logical_next_2 = 0;
+        RmOff logical_1 = 0, logical_2 = 0;
+        RmOff logical_next_1 = 0, logical_next_2 = 0;
 
-        RmOff physical_1 = rm_offset_get_from_fd(fd1, logical_current, &logical_next_1, &is_last_1);
-        RmOff physical_2 = rm_offset_get_from_fd(fd2, logical_current, &logical_next_2, &is_last_2);
+        RmOff physical_1 = rm_offset_get_from_fd(fd1, logical_current, &logical_next_1, &logical_1, &is_last_1);
+        RmOff physical_2 = rm_offset_get_from_fd(fd2, logical_current, &logical_next_2, &logical_2, &is_last_2);
 
         if(is_last_1 != is_last_2) {
             RM_RETURN(RM_LINK_NONE);
         }
 
-        if(is_last_1 && is_last_2 && at_least_one_checked) {
-            RM_RETURN(RM_LINK_REFLINK);
-        }
-
         if(physical_1 != physical_2) {
 #if _RM_OFFSET_DEBUG
             rm_log_debug_line("Physical offsets differ at byte %" G_GUINT64_FORMAT
                               ": %"G_GUINT64_FORMAT "<> %" G_GUINT64_FORMAT,
                               logical_current, physical_1, physical_2);
+#endif
+            RM_RETURN(RM_LINK_NONE);
+        }
+        if(logical_1 != logical_2) {
+#if _RM_OFFSET_DEBUG
+            rm_log_debug_line("Logical offsets differ after %" G_GUINT64_FORMAT
+                              " bytes: %" G_GUINT64_FORMAT "<> %" G_GUINT64_FORMAT,
+                              logical_current, logical_1, logical_2);
 #endif
             RM_RETURN(RM_LINK_NONE);
         }
@@ -1302,7 +1305,7 @@ RmLinkType rm_util_link_type(char *path1, char *path2) {
             RM_RETURN(RM_LINK_ERROR)
         }
 
-        if(logical_next_1 >= (RmOff)stat1.st_size) {
+        if(is_last_1) {
             /* phew, we got to the end */
 #if _RM_OFFSET_DEBUG
             rm_log_debug_line("Files are clones (share same data)")
@@ -1311,7 +1314,6 @@ RmLinkType rm_util_link_type(char *path1, char *path2) {
         }
 
         logical_current = logical_next_1;
-        at_least_one_checked = true;
     }
 
     RM_RETURN(RM_LINK_ERROR);
