@@ -16,8 +16,8 @@
 *
 * Authors:
 *
-*  - Christopher <sahib> Pahl 2010-2017 (https://github.com/sahib)
-*  - Daniel <SeeSpotRun> T.   2014-2017 (https://github.com/SeeSpotRun)
+*  - Christopher <sahib> Pahl 2010-2020 (https://github.com/sahib)
+*  - Daniel <SeeSpotRun> T.   2014-2020 (https://github.com/SeeSpotRun)
 *
 * Hosted on http://github.com/sahib/rmlint
 *
@@ -76,7 +76,7 @@ typedef struct stat RmStat;
 #define FLOAT_SIGN_DIFF(X, Y, tol) ((X) - (Y) > (tol) ? 1 : ((Y) - (X) > (tol) ? -1 : 0))
 
 // Time tolerance (seconds) when comparing two mtimes
-#define MTIME_TOL (0.00000001)
+#define MTIME_TOL (0.000001)
 
 #define RETURN_IF_NONZERO(X) \
     if((X) != 0) {           \
@@ -87,7 +87,7 @@ typedef struct stat RmStat;
 //       SYSCALL WRAPPERS         //
 ////////////////////////////////////
 
-static inline int rm_sys_stat(const char *path, RmStat *buf) {
+WARN_UNUSED_RESULT static inline int rm_sys_stat(const char *path, RmStat *buf)  {
 #if HAVE_STAT64 && !RM_IS_APPLE
     return stat64(path, buf);
 #else
@@ -95,7 +95,7 @@ static inline int rm_sys_stat(const char *path, RmStat *buf) {
 #endif
 }
 
-static inline int rm_sys_lstat(const char *path, RmStat *buf) {
+WARN_UNUSED_RESULT static inline int rm_sys_lstat(const char *path, RmStat *buf) {
 #if HAVE_STAT64 && !RM_IS_APPLE
     return lstat64(path, buf);
 #else
@@ -188,7 +188,7 @@ char *rm_util_get_username(void);
 char *rm_util_get_groupname(void);
 
 ////////////////////////////////////
-//       GENERAL UTILITES         //
+//       GENERAL UTILITIES         //
 ////////////////////////////////////
 
 #define RM_LIST_NEXT(node) ((node) ? node->next : NULL)
@@ -203,7 +203,7 @@ char *rm_util_strsub(const char *string, const char *subs, const char *with);
 /**
  * @brief Check if a file has an invalid gid/uid or both.
  *
- * @return the appropiate RmLintType for the file
+ * @return the appropriate RmLintType for the file
  */
 int rm_util_uid_gid_check(RmStat *statp, RmUserList *userlist);
 
@@ -408,7 +408,7 @@ bool rm_mounts_is_evil(RmMountTable *self, dev_t to_check);
 bool rm_mounts_can_reflink(RmMountTable *self, dev_t source, dev_t dest);
 
 /////////////////////////////////
-//    FIEMAP IMPLEMENATION     //
+//    FIEMAP IMPLEMENTATION     //
 /////////////////////////////////
 
 /**
@@ -416,7 +416,7 @@ bool rm_mounts_can_reflink(RmMountTable *self, dev_t source, dev_t dest);
  *
  * @return the physical offset starting from the disk.
  */
-RmOff rm_offset_get_from_fd(int fd, RmOff file_offset, RmOff *file_offset_next, bool *is_last);
+RmOff rm_offset_get_from_fd(int fd, RmOff file_offset, RmOff *file_offset_next, RmOff *logical_offset, bool *is_last);
 
 /**
  * @brief Lookup the physical offset of a file path at any given offset.
@@ -515,4 +515,9 @@ gdouble rm_running_mean_get(RmRunningMean *m);
  */
 void rm_running_mean_unref(RmRunningMean *m);
 
+/**
+ * @brief See GLib docs for g_canonicalize_filename().
+ */
+gchar *rm_canonicalize_filename(const gchar *filename, const gchar *relative_to);
+
 #endif /* RM_UTILITIES_H_INCLUDE*/
