@@ -37,3 +37,19 @@ def test_simple():
     assert cksum_1 != '0' * 32
     assert cksum_2 != '0' * 32
     assert cksum_3 != '0' * 32
+
+
+# regression test for GitHub issue #496
+@with_setup(usual_setup_func, usual_teardown_func)
+def test_no_checksum():
+    # rmlint will not (normally) hash files with no same-sized siblings
+    create_file('x', 'a')
+    create_file('yy', 'b')
+
+    # test for 'free(): invalid pointer' crash
+    head, *data, foot, csv = run_rmlint('-S a -c csv:unique', outputs=['csv'])
+    assert not data
+
+    # empty checksums should make it to output
+    csv_data = csv_string_to_data(csv)
+    assert [r[3] for r in csv_data] == ['', '']
