@@ -9,22 +9,6 @@ from contextlib import contextmanager
 from tests.utils import *
 
 
-@contextmanager
-def assert_exit_code(status_code):
-    """
-    Assert that the with block yields a subprocess.CalledProcessError
-    with a certain return code. If nothing is thrown, status_code
-    is required to be 0 to survive the test.
-    """
-    try:
-        yield
-    except subprocess.CalledProcessError as exc:
-        assert exc.returncode == status_code
-    else:
-        # No exception? status_code should be fine.
-        assert status_code == 0
-
-
 @with_setup(usual_setup_func, usual_teardown_func)
 def test_equal_files():
     path_a = create_file('1234', 'a')
@@ -154,3 +138,22 @@ def test_equal_empty_files_or_other_lint():
             '--equal', path_a, path_b,
             use_default_dir=False,
         )
+
+
+# regression test for GitHub issue #552
+@with_setup(usual_setup_func, usual_teardown_func)
+def test_default_outputs_disabled():
+    create_file('xxx', 'a')
+    create_file('xxx', 'b')
+
+    cwd = os.getcwd()
+    try:
+        os.chdir(TESTDIR_NAME)
+        run_rmlint('--equal a b', use_default_dir=False, with_json=False)
+
+        # Users of --equal, including our own sh format, do not expect to
+        # create or overwrite rmlint.sh or rmlint.json.
+        assert not os.path.exists('rmlint.sh')
+        assert not os.path.exists('rmlint.json')
+    finally:
+        os.chdir(cwd)
