@@ -1,19 +1,25 @@
 #!/usr/bin/env python3
 # encoding: utf-8
 from nose import with_setup
+from parameterized import parameterized
 from tests.utils import *
 
 
 def filter_part_of_directory(data):
-    return [entry for entry in data if entry["type"] != "part_of_directory"]
+    data = [entry for entry in data if entry["type"] != "part_of_directory"]
+    data.sort(key=lambda entry: (entry['path'],) if entry['type'] == 'unique_file' else ())
+    return data
 
+
+# --write-unfinished variant is a regression test for GitHub issue #562
+@parameterized([((),), (('--write-unfinished',),)])
 @with_setup(usual_setup_func, usual_teardown_func)
-def test_simple():
+def test_simple(extra_opts):
     create_file('xxx', '1/a')
     create_file('xxx', '2/a')
     create_file('xxx', 'a')
 
-    head, *data, footer = run_rmlint('-p -D --rank-by A')
+    head, *data, footer = run_rmlint('-p -D --rank-by A', *extra_opts)
     data = filter_part_of_directory(data)
 
     assert 2 == sum(find['type'] == 'duplicate_dir' for find in data)
@@ -437,13 +443,21 @@ def test_equal_content_different_layout():
     create_file('xxx', "tree-b/x")
     create_file('yyy', "tree-b/y")
 
-    head, *data, footer = run_rmlint('-p -D --rank-by a')
-    data = filter_part_of_directory(data)
+    # Test all checksum types, even outside of pedantic mode.
+    # That allows us to test for regressions in the cumulative digest.
+    options = ['-p']
+    if not get_env_flag('RM_TS_PEDANTIC'):
+        for cksum_type in CKSUM_TYPES:
+            options.append('--algorithm=' + cksum_type)
 
-    assert data[0]["path"].endswith("tree-a")
-    assert data[0]["is_original"] is True
-    assert data[1]["path"].endswith("tree-b")
-    assert data[1]["is_original"] is False
+    for option in options:
+        head, *data, footer = run_rmlint('-D --rank-by a', option)
+        data = filter_part_of_directory(data)
+
+        assert data[0]["path"].endswith("tree-a")
+        assert data[0]["is_original"] is True
+        assert data[1]["path"].endswith("tree-b")
+        assert data[1]["is_original"] is False
 
     # Now, try to honour the layout
     head, *data, footer = run_rmlint('-p -Dj --rank-by a')
