@@ -9,15 +9,20 @@ import os
 import json
 import time
 import pprint
+import psutil
+import re
 import shutil
 import shlex
 import struct
 import subprocess
 import contextlib
 import xattr
+from nose.plugins.skip import SkipTest
+from nose.tools import make_decorator
 
 
 TESTDIR_NAME = os.getenv('RM_TS_DIR') or '/tmp/rmlint-unit-testdir'
+TESTDIR_NAME = os.path.realpath(TESTDIR_NAME)
 
 CKSUM_TYPES = [
     'murmur',
@@ -42,6 +47,9 @@ CKSUM_TYPES = [
     'paranoid',
 ]
 
+_REFLINK_CAPABLE_FILESYSTEMS = {'btrfs', 'xfs', 'ocfs2'}
+
+
 def runs_as_root():
     return os.geteuid() == 0
 
@@ -104,7 +112,9 @@ def run_rmlint_once(*args,
                     with_json=True,
                     directly_return_output=False,
                     use_shell=False,
-                    verbosity="-V"):
+                    verbosity="-V",
+                    check=True,
+                    timeout=None):
     if use_default_dir:
         if dir_suffix:
             target_dir = os.path.join(TESTDIR_NAME, dir_suffix)
@@ -132,7 +142,7 @@ def run_rmlint_once(*args,
 
     cmd += shlex.split(' '.join(args))
     if with_json:
-        cmd += ['-o', 'json:/tmp/out.json', '-c', 'json:oneline']
+        cmd += ['-o', 'json:' + os.path.join(TESTDIR_NAME, 'out.json'), '-c', 'json:oneline']
 
     for idx, output in enumerate(outputs or []):
         cmd.append('-o')
@@ -157,17 +167,22 @@ def run_rmlint_once(*args,
         # Use /bin/bash, not /bin/sh
         cmd = ["/bin/bash", "-c", " ".join(cmd)]
 
-    output = subprocess.check_output(cmd, env=env)
+    result = subprocess.run(cmd, env=env, stdout=subprocess.PIPE, stderr=subprocess.PIPE, timeout=timeout)
+    print(result.stderr.decode('utf-8'), end='')
+
     if get_env_flag('RM_TS_USE_GDB'):
         print('==> START OF GDB OUTPUT <==')
-        print(output.decode('utf-8'))
+        print(result.stdout.decode('utf-8'), end='')
         print('==> END OF GDB OUTPUT <==')
 
+    if check:
+        result.check_returncode()
+
     if directly_return_output:
-        return output
+        return result.stdout if check else (result, result.stdout)
 
     if with_json:
-        with open('/tmp/out.json', 'r') as f:
+        with open(os.path.join(TESTDIR_NAME, 'out.json'), 'r') as f:
             json_data = json.loads(f.read())
     else:
         json_data = []
@@ -176,10 +191,9 @@ def run_rmlint_once(*args,
     for idx, output in enumerate(outputs or []):
         with open(os.path.join(TESTDIR_NAME, '.' + output + '-' + str(idx)), 'r', encoding='utf8') as handle:
             read_outputs.append(handle.read())
-    if outputs is None:
-        return json_data
-    else:
-        return json_data + read_outputs
+    if outputs:
+        json_data.extend(read_outputs)
+    return json_data if check else (result, json_data)
 
 
 def compare_json_doc(doc_a, doc_b, compare_checksum=False):
@@ -401,3 +415,68 @@ def must_read_xattr(path):
           See create_special_fs for a workaround.
     """
     return dict(xattr.xattr(os.path.join(TESTDIR_NAME, path)).items())
+
+
+@contextlib.contextmanager
+def assert_exit_code(status_code):
+    """
+    Assert that the with block yields a subprocess.CalledProcessError
+    with a certain return code. If nothing is thrown, status_code
+    is required to be 0 to survive the test.
+    """
+    try:
+        yield
+    except subprocess.CalledProcessError as exc:
+        assert exc.returncode == status_code
+    else:
+        # No exception? status_code should be fine.
+        assert status_code == 0
+
+
+def _up(path):
+    while path:
+        yield path
+        if path == "/":
+            break
+        path = os.path.dirname(path)
+
+
+def is_on_reflink_fs(path):
+    parts = psutil.disk_partitions(all=True)
+
+    # iterate up from `path` until mountpoint found
+    for up_path in _up(path):
+        for part in parts:
+            if up_path == part.mountpoint:
+                print("{0} is {1} mounted at {2}".format(path, part.fstype, part.mountpoint))
+                return (part.fstype in _REFLINK_CAPABLE_FILESYSTEMS)
+
+    print("No mountpoint found for {}".format(path))
+    return False
+
+
+# decorator for tests dependent on reflink-capable testdir
+def needs_reflink_fs(test):
+    def no_support(*args):
+        raise SkipTest("btrfs not supported")
+
+    def not_reflink_fs(*args):
+        raise SkipTest("testdir is not on reflink-capable filesystem")
+
+    if not has_feature('btrfs-support'):
+        return make_decorator(test)(no_support)
+    elif not is_on_reflink_fs(TESTDIR_NAME):
+        return make_decorator(test)(not_reflink_fs)
+    else:
+        return test
+
+
+# count the number of line in a file which start with each pattern
+def pattern_count(path, patterns):
+    counts = [0] * len(patterns)
+    with open(path, 'r') as f:
+        for line in f:
+            for i, pattern in enumerate(patterns):
+                if re.match(pattern, line):
+                    counts[i] += 1
+    return counts
